/* 
This file is part of the PolePosition database benchmark
http://www.polepos.org

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package org.polepos.teams.hibernate;

import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.polepos.framework.Car;
import org.polepos.framework.CarMotorFailureException;
import org.polepos.framework.CheckSummable;
import org.polepos.framework.Driver;
import org.polepos.framework.TurnSetup;

/**
 * @author Herkules
 */
public abstract class HibernateDriver extends Driver
{

	public void takeSeatIn( Car car, TurnSetup setup ) throws CarMotorFailureException{
        super.takeSeatIn(car, setup);
	}

	public void prepare() throws CarMotorFailureException {
        hibernateCar().openSession();
	}
	
	public void backToPit(){
        hibernateCar().closeSession();
	}
    
    public HibernateCar hibernateCar(){
        return (HibernateCar)car();
    }
		
	public Session db(){
		return hibernateCar().getSession();
	}

	protected void doQuery(String query) {
        
		try{
			List list = db().createCriteria(query).list();
			for(int i = 0; i < list.size(); i++) {
				Object o = list.get(i);
                if(o instanceof CheckSummable){
                    addToCheckSum(((CheckSummable)o).checkSum());
                }
			}
		}
		catch ( HibernateException hex ){
			hex.printStackTrace();
		}		
	}
	
	protected void doSingleResultQuery(String query) {
		try{
			
			Object o = db().createCriteria(query).uniqueResult();
            if(o instanceof CheckSummable){
                addToCheckSum(((CheckSummable)o).checkSum());
            }			
		}
		catch ( HibernateException hex ){
			System.err.println("Multiple results, expected only one.");
			hex.printStackTrace();
		}		
	}
}
