package org.polepos.teams.hibernate;


import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Transaction;
import org.polepos.circuits.matao.MataoDriver;
import org.polepos.teams.hibernate.data.H3Department;
import org.polepos.teams.hibernate.data.H3Employee;
import org.polepos.teams.hibernate.data.H3Engineer;
import org.polepos.teams.hibernate.data.H3Manager;
import org.polepos.teams.hibernate.data.H3ProfessionalUnion;


public class MataoHibernate extends HibernateDriver implements MataoDriver {


	public void consulta_relacionamento_recursivo() {

		int niveis = setup().getSelectDepth();
		String managedNivel = "manager" + (niveis - 1);
		String queryRecursiva = "select " + managedNivel +  " from H3Employee employee inner join employee.managedEmployees as manager0";

		for(int i = 1; i < niveis; i++) {
			queryRecursiva += " inner join manager"+(i-1)+".managedEmployees as manager"+i;
		}

		queryRecursiva += " where employee.manager is NULL";

		List<H3Employee> q = db().createQuery(queryRecursiva).setFirstResult(1).list();
		System.err.println("consulta_relacionamento_recursivo: " +q.size());
		
	}

	public void consulta_relacionamento_recursivo_hot() {
		consulta_relacionamento_recursivo();

	}

	public void insercao_gerente() {

		int numobjects = setup().getObjectCount();
		int commitintervall = setup().getCommitInterval();
		int commitctr = 0;
		try{
			Transaction tx = db().beginTransaction();

			H3Department department = (H3Department) db().createCriteria(H3Department.class).list().get(0);

			for (int i = 1; i<= numobjects; i++) {
				H3Manager manager = new H3Manager();                
				manager.setDepartment(department);
				manager.setName("Gerente"+i);
				db().save(manager);

				addToCheckSum(i);
				if ( commitintervall > 0  &&  ++commitctr >= commitintervall )
				{
					commitctr = 0;
					tx.commit();
					tx = db().beginTransaction();
					Log.logger.fine( "commit while writing at " + i+1 ); //NOI18N
				}
			}
			tx.commit();
		}
		catch ( HibernateException hex ){
			hex.printStackTrace();
		}

	}

	public void insercao_relacionamento_mn() {
		int numobjects = setup().getObjectCount();
		int commitintervall = setup().getCommitInterval();
		int commitctr = 0;
		try{
			Transaction tx = db().beginTransaction();

			H3Department department = (H3Department)db().createCriteria(H3Department.class).list().get(0);
			H3ProfessionalUnion professionalUnion = new H3ProfessionalUnion();
			professionalUnion.setName("CUT");
			db().save(professionalUnion);


			for (int i = 1; i<= numobjects/2; i++) {
				H3Engineer engineer = new H3Engineer();                
				engineer.setName("João"+i);
				engineer.setDepartment(department);
				engineer.getProfessionalUnions().add(professionalUnion);  
				professionalUnion.getEngineers().add(engineer); 
				db().save(engineer);

				addToCheckSum(i);
				if ( commitintervall > 0  &&  ++commitctr >= commitintervall )
				{
					commitctr = 0;
					tx.commit();
					tx = db().beginTransaction();
					Log.logger.fine( "commit while writing at " + i+1 ); //NOI18N
				}
			}
			tx.commit();
		}
		catch ( HibernateException hex ){
			hex.printStackTrace();
		}

	}

	public void buscaProfundidadeModelo() {
		List q = db().createQuery("select desktop " +
				"from H3Desktop desktop " +
				"inner join desktop.secretary as secretary" +
				" inner join secretary.department as department" +
				" inner join department.division as division" +
		" where division.boss.id = '1'" ).list();


		System.err.println("busca em profundidade no modelo: "+q.size());		
	}

	public void buscaProfundidadeModelo_hot() {
		buscaProfundidadeModelo();
	}

	public void consulta_relacionamento_mn() {
		List q = db().createQuery("select engineer " +
				"from H3Engineer engineer " +
				"inner join engineer.professionalUnions as assoc" +				
		" where assoc.name = 'CUT' and engineer.id < 99" ).setFirstResult(1).list();


		System.err.println("consulta em relacionamento M-N: "+q.size());		
	}

	public void insercaoArvoreModelo() {

		try {

			Transaction tx = db().beginTransaction();

			H3Department department = (H3Department)db().createCriteria(H3Department.class).list().get(0);

			H3Employee employee = insercaoArvoreAux(setup().getTreeDepth(),department);

			employee.setDepartment(department);

			db().save(employee);

			tx.commit();

		}

		catch ( HibernateException hex ){
			hex.printStackTrace();
		}

	}


	private H3Employee insercaoArvoreAux(int n, H3Department department) {

		if(n == 1) {
			H3Employee employee = new H3Employee();
			employee.setDepartment(department);
			employee.setName("Joao"+n);
			return employee;
		}

		else {

			H3Employee employee = new H3Employee();
			employee.setDepartment(department);
			employee.setName("Joao"+n);

			H3Employee filho1 = insercaoArvoreAux(n - 1,department);
			H3Employee filho2 = insercaoArvoreAux(n - 1,department);

			filho1.setManager(employee);
			filho2.setManager(employee);

			employee.getManagedEmployees().add(filho1);
			employee.getManagedEmployees().add(filho2);

			return employee;
		}

	}


}
