/* 
This file is part of the PolePosition database benchmark
http://www.polepos.org

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package org.polepos.teams.jdbc;



import java.sql.ResultSet;
import java.sql.SQLException;

import org.polepos.circuits.matao.MataoDriver;
import org.polepos.data.Pilot;
import org.polepos.framework.Car;
import org.polepos.framework.CarMotorFailureException;
import org.polepos.framework.TurnSetup;
import org.polepos.teams.hibernate.Log;
import org.polepos.teams.jdbc.drivers.melbourne.BulkWriteMultiValue;
import org.polepos.teams.jdbc.drivers.melbourne.BulkWritePreparedStatement;
import org.polepos.teams.jdbc.drivers.melbourne.BulkWriteStrategy;
import org.polepos.teams.jpa.data.modelo.Engenheiro;
import org.polepos.teams.jpa.data.modelo.Funcionario;
import org.polepos.teams.jpa.data.modelo.Gerente;

/**
 * @author Herkules
 */
public class MataoJdbc extends JdbcDriver implements MataoDriver
{
	/**
	 * Number of pilot to be written at once.
	 */
	private final static int BULKSIZE = 1000;
	private static long position = 0;
	
	public void takeSeatIn(Car car, TurnSetup setup) throws CarMotorFailureException{
		super.takeSeatIn(car, setup);

		jdbcCar().openConnection();

//
//		Create database structure
//
//		jdbcCar().dropTable( "australia" );
//		jdbcCar().createTable( "australia", new String[]{ "ID", "Name", "FirstName", "Points", "LicenseID" }, 
//		new Class[]{Integer.TYPE, String.class, String.class, Integer.TYPE, Integer.TYPE} );

		jdbcCar().closeConnection();
	}

	public void write(){

		int numobjects = setup().getObjectCount(); 
		int commitintervall = setup().getCommitInterval();
		int commitctr = 0;

		Pilot[] pilots = new Pilot[ 100 ];
		int idx = 0;

		//BulkWriteStrategy writer = new BulkWriteSingle();
		BulkWriteStrategy writer = new BulkWritePreparedStatement( jdbcCar(), "australia" );
		//BulkWriteStrategy writer = new BulkWriteMultiValue();

		for ( int i = 1; i <= numobjects; i++ )
		{
			Pilot p = new Pilot( "Pilot_" + i, "Herkules", i, i );
			pilots[ idx++ ] = p;
			if ( idx == BULKSIZE )
			{
				writer.savePilots("australia", pilots, idx, i - idx + 1 );
				idx = 0;
			}

			if ( commitintervall > 0  &&  ++commitctr >= commitintervall )
			{
				commitctr = 0;
				jdbcCar().commit();
				Log.logger.fine ( "commit while writing at " + i+1 ); //NOI18N
			}

			addToCheckSum(i);
		}

		// Write the rest
		writer.savePilots("australia", pilots, idx, numobjects - idx );

		jdbcCar().commit();
	}

	public void read(){

		int numobjects = setup().getObjectCount();

		try{
			ResultSet rs = jdbcCar().executeQuery("select * from australia");

			for ( int i = 0; i < numobjects; i++ ){
				rs.next();
				Pilot p = new Pilot( rs.getString( 2 ), rs.getString( 3 ), rs.getInt( 4 ), rs.getInt( 5 ) );
				addToCheckSum(p.getPoints());
			}
		}
		catch ( SQLException sqlex ){
			sqlex.printStackTrace();
		}
	}

	public void read_hot() {
		read();
	}

	public void delete(){
		jdbcCar().executeSQL( "delete from australia" );
		jdbcCar().commit();
	}

	public void consulta_relacionamento_recursivo() {
		// TODO Auto-generated method stub
		int niveis = setup().getSelectDepth();
		int size;
		String managedNivel = "funcionario" + (niveis - 1);
		String queryRecursiva = "select " + managedNivel +  " from funcionariojdbc funcionario inner join funcionario.funcionarios as funcionario0";

		for(int i = 1; i < niveis; i++) {
			queryRecursiva += " inner join funcionario"+(i-1)+".funcionarios as funcionario"+i;
		}

		queryRecursiva += " where funcionario.superior is NULL";

		ResultSet rs = jdbcCar().executeQuery(queryRecursiva);


		size = 0;
		try {
			while(rs.next()) {
				size++;
			}
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		System.err.println("consulta_relacionamento_recursivo: " + size );

	}

	public void consulta_relacionamento_recursivo_hot() {
		consulta_relacionamento_recursivo();

	}

	public void insercao_gerente() {
		int numobjects = setup().getObjectCount(); 
		int commitintervall = setup().getCommitInterval();
		int commitctr = 0;

		Funcionario[] gerentes = new Gerente[ BULKSIZE ];
		int idx = 0;

		//BulkWriteStrategy writer = new BulkWriteSingle();
		BulkWriteMultiValue writer = new BulkWriteMultiValue( jdbcCar());
		//BulkWriteStrategy writer = new BulkWriteMultiValue();
		ResultSet rs = jdbcCar().executeQuery("SELECT max(id) FROM funcionariojdbc f order by id;");

		try {
			rs.next();
			Funcionario.setIdGenerator(rs.getInt(1));

		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}


		for ( int i = 1; i <= numobjects; i++ )
		{

			Funcionario p = new Gerente();
			p.setNome("Gerente"+i);
			gerentes[ idx++ ] = p;
			if ( idx == BULKSIZE )
			{
				writer.saveFuncionarios( gerentes, idx, "Gerente");
				idx = 0;
			}

			if ( commitintervall > 0  &&  ++commitctr >= commitintervall )
			{
				commitctr = 0;
				jdbcCar().commit();
				Log.logger.fine ( "commit while writing at " + i+1 ); //NOI18N
			}

			addToCheckSum(i);
		}

		// Write the rest
		writer.saveFuncionarios( gerentes, idx, "Gerente" );

		jdbcCar().commit();

	}

	public void buscaProfundidadeModelo() {
		// TODO Auto-generated method stub

	}

	public void buscaProfundidadeModelo_hot() {
		// TODO Auto-generated method stub

	}

	public void consulta_relacionamento_mn() {
		// TODO Auto-generated method stub

	}

	public void insercaoArvoreModelo() {

		int treeDepth = setup().getTreeDepth(); 
		StringBuffer stmt = new StringBuffer( "insert into funcionariojdbc (id,dtype,nome,departamento_id) values ");
//		int[] fieldSuperior_id = new int[((int)Math.pow(2, treeDepth))-1];
//		int[] fieldId = new int[((int)Math.pow(2, treeDepth))-1];
//		int[] fieldId2 = new int[((int)Math.pow(2, treeDepth))-1];
		
		
		
		
			
			
			position = 200000+((int)Math.pow(2, treeDepth))+10;

		
				
		insercaoArvoreAux(stmt,treeDepth);
		String sql  = stmt.toString();
		jdbcCar().executeSQL(sql.substring(0,sql.length()-1));
//		int index = 0;
		

//			try {
//				int size = fieldId.length;
//				for(int i = 0; i < size; i++) {
//					if(index < size) {
//						mStmt.setInt(1, fieldSuperior_id[index]);
//						mStmt.setInt(2, fieldId[index]);
//						mStmt.setInt(3, fieldId2[index]);
////						System.err.println("superior:"+fieldSuperior_id[index]+" filho1:"+fieldId[index]+" filho2:"+fieldId2[index]);
//						++index;
//						mStmt.executeUpdate();
//					}
//					else
//						break;
//					
//				}
//			} catch (SQLException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}

	}


	private long insercaoArvoreAux(StringBuffer sql,int n) {

		if(n == 0) {
			sql.append( "(" ).append( ++position   ).append( ", '" );
			sql.append( "Funcionario" ).append( "', '" );
			sql.append( "Joao"+n ).append( "', " );
			sql.append( Integer.toString( 1 ) ).append( ")," );
			return position;
		}

		else {

			
			long filho1 = insercaoArvoreAux(sql,n - 1);
			long filho2 = insercaoArvoreAux(sql,n - 1);
				
//			fieldSuperior_id[position] = id;
//			fieldId[position] = filho1;
//			fieldId2[position] = filho2;
//			++position;
			sql.append( "(" ).append(  ++position  ).append( ", '" );
			sql.append( "Funcionario" ).append( "', '" );
			sql.append( "Joao"+n ).append( "', " );
			sql.append( Integer.toString( 1 ) ).append( ")," );
			
			return position;
		}

	}

	private void values( StringBuffer to, Funcionario p, int idx,String dtype )
	{
		to.append( "(" ).append( Integer.toString( idx ) ).append( ", '" );
		to.append( dtype ).append( "', '" );
		to.append( p.getNome() ).append( "', " );
		to.append( Integer.toString( 1 ) ).append( ")," );	
	}


	public void insercao_relacionamento_mn() {
		int numobjects = setup().getObjectCount();
		int commitintervall = setup().getCommitInterval();
		int commitctr = 0;
		int idx = 0;
		int professionalunion_id = 0;
		ResultSet rs = jdbcCar().executeQuery("SELECT max(id) FROM professionalunionjdbc p order by id;");


		try {
			rs.next();
			professionalunion_id = rs.getInt(1)+1;
		} catch (SQLException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		StringBuffer stmt = new StringBuffer( "insert into professionalunionjdbc");
		stmt.append(" (id,nome) values (");
		stmt.append( professionalunion_id ).append( ", '" );
		stmt.append( "CUT" ).append( "')" );
		jdbcCar().executeSQL( stmt.toString() );

		Funcionario[] engenheiros = new Engenheiro[ BULKSIZE ];

		//BulkWriteStrategy writer = new BulkWriteSingle();
		BulkWriteMultiValue writer = new BulkWriteMultiValue( jdbcCar() );
		//BulkWriteStrategy writer = new BulkWriteMultiValue();
		rs = jdbcCar().executeQuery("SELECT max(id) FROM funcionariojdbc f order by id;");

		try {
			rs.next();
			Funcionario.setIdGenerator(rs.getInt(1));

		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		for ( int i = 1; i <= numobjects; i++ )
		{

			Funcionario p = new Engenheiro();
			p.setNome("Engenheiro"+i);
			engenheiros[ idx++ ] = p;
			if ( idx == BULKSIZE )
			{
				writer.saveFuncionarios( engenheiros, idx,professionalunion_id, "Engenheiro");
				idx = 0;
			}

			if ( commitintervall > 0  &&  ++commitctr >= commitintervall )
			{
				commitctr = 0;
				jdbcCar().commit();
				Log.logger.fine ( "commit while writing at " + i+1 ); //NOI18N
			}

			addToCheckSum(i);
		}

		// Write the rest
		writer.saveFuncionarios( engenheiros, idx,professionalunion_id, "Engenheiro" );

		jdbcCar().commit();

	}

	public static long getPosition() {
		return position;
	}

	public static void setPosition(long position) {
		MataoJdbc.position = position;
	}



}
