/* 
This file is part of the PolePosition database benchmark
http://www.polepos.org

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package org.polepos.teams.jdbc.drivers.melbourne;

import java.sql.PreparedStatement;
import java.sql.SQLException;

import org.polepos.data.Pilot;
import org.polepos.teams.jdbc.JdbcCar;
import org.polepos.teams.jpa.data.modelo.Funcionario;

/**
 *
 * @author Herkules
 *
 * Typical:
 * 50000 objects: write 4703ms (0.09406ms/object), read 172ms (0.00344ms/object), delete 0ms (0.0ms/object)
 */
public class BulkWritePreparedStatement implements BulkWriteStrategy
{
	private final PreparedStatement mStmt;
	private final PreparedStatement mStmt2;

	/** 
	 * Creates a new instance of BulkWriteSingle.
	 */
	public BulkWritePreparedStatement( JdbcCar car, String tablename )
	{
		mStmt = car.prepareStatement( "insert into " + tablename + " (id, dtype,nome,departamento_id) values (?,?,?,?)" );
		mStmt2 = car.prepareStatement( "insert into funcionario_professionalunionjdbc (engenheiros_id, professionalunions_id) values (?,?)" );
	}

	/**
	 * Dump an array of pilots to the DB by writing one-by-one.
	 */
	public void savePilots(String tablename, Pilot[] p, int count, int index )
	{
		try
		{
			for ( int i = 0; i < count; i++ )
			{
				savePilot(p[i], index++ );
			}		
			
			// mckoi complains with an exception if the batch is empty
			if ( count > 0 )
			{
				mStmt.executeBatch();
			}
		}
		catch ( SQLException sqlex )
		{
            sqlex.printStackTrace();			
		}
	}
	
	
    /**
	 * Helper:
     * Write a single pilot to the database.
     */
    private void savePilot( Pilot p, int index ) throws SQLException
    {
		mStmt.setInt(		1, index );
		mStmt.setString(	2, p.getName() );
		mStmt.setString(	3, p.getFirstName() );
		mStmt.setInt(		4, p.getPoints() );
		mStmt.setInt(		5, p.getPoints() );
		mStmt.addBatch();
    }
    
    public void saveFuncionarios(String tablename, Funcionario[] p, int count ,String dtype)
	{
		try
		{
			for ( int i = 0; i < count; i++ )
			{
				saveFuncionario(p[i], Funcionario.getIdGenerator(), dtype);
			}		
			
			// mckoi complains with an exception if the batch is empty
			if ( count > 0 )
			{
				mStmt.executeBatch();
			}
		}
		catch ( SQLException sqlex )
		{
            sqlex.printStackTrace();			
		}
	}
	
    
    public void saveFuncionarios(String tablename, Funcionario[] p, int count, int professionalunion_id ,String dtype)
	{
		try
		{
			for ( int i = 0; i < count; i++ )
			{	
				long id = Funcionario.getIdGenerator();
				saveFuncionario(p[i], id, dtype);
				saveEngenheiroProfessionalUnion(id, professionalunion_id);
			}		
			
			// mckoi complains with an exception if the batch is empty
			if ( count > 0 )
			{
				mStmt.executeBatch();
				mStmt2.executeBatch();
			}
		}
		catch ( SQLException sqlex )
		{
            sqlex.printStackTrace();			
		}
	}
    
	
    /**
	 * Helper:
     * Write a single pilot to the database.
     */
    private void saveFuncionario( Funcionario p, long index, String dtype ) throws SQLException
    {
		mStmt.setLong(1, index );
		mStmt.setString(2, dtype);
		mStmt.setString(3, p.getNome());
		mStmt.setInt(4, 1);
		mStmt.addBatch();
    }
    
        
    private void saveEngenheiroProfessionalUnion( long engenheiro_id,long professionaunion_id ) throws SQLException
    {
		mStmt2.setLong(1, engenheiro_id );
		mStmt2.setLong(2, professionaunion_id);
		mStmt2.addBatch();
    }
}
