package org.polepos.teams.jpa;

import java.util.List;

import javax.persistence.EntityTransaction;

import org.polepos.circuits.matao.MataoDriver;
import org.polepos.teams.hibernate.Log;
import org.polepos.teams.jpa.data.modelo.Departamento;
import org.polepos.teams.jpa.data.modelo.Engenheiro;
import org.polepos.teams.jpa.data.modelo.Funcionario;
import org.polepos.teams.jpa.data.modelo.Gerente;
import org.polepos.teams.jpa.data.modelo.ProfessionalUnion;

public class MataoJPA extends JPADriver implements MataoDriver {


	public void consulta_relacionamento_recursivo() {
		// TODO Auto-generated method stub
		int niveis = setup().getSelectDepth();
		String managedNivel = "funcionario" + (niveis - 1);
		String queryRecursiva = "select " + managedNivel +  " from Funcionario funcionario inner join funcionario.funcionarios as funcionario0";

		for(int i = 1; i < niveis; i++) {
			queryRecursiva += " inner join funcionario"+(i-1)+".funcionarios as funcionario"+i;
		}

		queryRecursiva += " where funcionario.superior is NULL";

		List<Funcionario> q = db().createQuery(queryRecursiva).setFirstResult(1).getResultList();
		System.err.println("consulta_relacionamento_recursivo: " +q.size());
		
	}

	public void consulta_relacionamento_recursivo_hot() {
		consulta_relacionamento_recursivo();

	}

	public void insercao_gerente() {
		int numobjects = setup().getObjectCount();
		int commitintervall = setup().getCommitInterval();
		int commitctr = 0;
		try{
			EntityTransaction tx = db().getTransaction();
			tx.begin();
			Departamento departamento = (Departamento)db().createQuery("SELECT d FROM Departamento d").getResultList().get(0);

			for (int i = 1; i<= numobjects; i++) {
				Gerente gerente = new Gerente();                
				gerente.setDepartamento(departamento);
				gerente.setNome("Gerente"+i);
				db().persist(gerente);

				addToCheckSum(i);
				if ( commitintervall > 0  &&  ++commitctr >= commitintervall )
				{
					commitctr = 0;
					tx.commit();
					tx.begin();
					Log.logger.fine( "commit while writing at " + i+1 ); //NOI18N
				}
			}
			tx.commit();

		}
		catch ( Exception hex ){
			hex.printStackTrace();
		}
	}

	public void insercao_relacionamento_mn() {
		int numobjects = setup().getObjectCount();
		int commitintervall = setup().getCommitInterval();
		int commitctr = 0;
		try{
			EntityTransaction tx = db().getTransaction();
			tx.begin();

			Departamento departamento = (Departamento)db().createQuery("SELECT d FROM Departamento d").getResultList().get(0);
			ProfessionalUnion professionalUnion = new ProfessionalUnion();
			professionalUnion.setNome("CUT");
			db().persist(professionalUnion);

			int count = setup().getObjectCount(); 
			for (int i = 1; i<= count/2; i++) {            	
				Engenheiro engenheiro = new Engenheiro();               
				engenheiro.setNome("João"+i);
				engenheiro.setDepartamento(departamento);
				engenheiro.getProfessionalUnions().add(professionalUnion);
				professionalUnion.getEngenheiros().add(engenheiro);
				db().persist(engenheiro);

				addToCheckSum(i);
				if ( commitintervall > 0  &&  ++commitctr >= commitintervall )
				{
					commitctr = 0;
					tx.commit();
					tx.begin();
					Log.logger.fine( "commit while writing at " + i+1 ); //NOI18N
				}
			}
			tx.commit();

		}
		catch ( Exception hex ){
			hex.printStackTrace();
		}
	}


	public void insercaoArvoreModelo() {

		try {

			EntityTransaction tx = db().getTransaction();
			tx.begin();

			Departamento departamento = (Departamento)db().createQuery("SELECT d FROM Departamento d").getResultList().get(0);

			Funcionario funcionario = insercaoArvoreAux(setup().getTreeDepth(),departamento);

			funcionario.setDepartamento(departamento);

			db().persist(funcionario);

			tx.commit();

		}

		catch ( Exception hex ){
			hex.printStackTrace();
		}

	}


	private Funcionario insercaoArvoreAux(int n, Departamento departamento) {

		if(n == 1) {
			Funcionario funcionario = new Funcionario();
			funcionario.setDepartamento(departamento);
			funcionario.setNome("Joao"+n);
			return funcionario;
		}

		else {

			Funcionario funcionario = new Funcionario();
			funcionario.setDepartamento(departamento);
			funcionario.setNome("Joao"+n);

			Funcionario filho1 = insercaoArvoreAux(n - 1,departamento);
			Funcionario filho2 = insercaoArvoreAux(n - 1,departamento);

			filho1.setSuperior(funcionario);
			filho2.setSuperior(funcionario);

			funcionario.getFuncionarios().add(filho1);
			funcionario.getFuncionarios().add(filho2);

			return funcionario;
		}

	}


	public void buscaProfundidadeModelo() {

		//List q = db().createQuery("select secretario.departamento from Secretario secretario" ).getResultList();

		List<Funcionario> q = db().createQuery("select desktop " +
				"from Desktop desktop" +
				" inner join desktop.secretario as secretario" +
				" inner join secretario.departamento as departamento" +
				" inner join departamento.divisao as divisao" +
				" where divisao.chefe.id = '1'" 				
		).getResultList();


		System.err.println("busca em profundidade no modelo: "+q.size());	

	}

	public void buscaProfundidadeModelo_hot() {
		buscaProfundidadeModelo();

	}

	public void consulta_relacionamento_mn() {
		List<Funcionario> q = db().createQuery("select engenheiro " +
				"from Engenheiro engenheiro " +
				"inner join engenheiro.professionalUnions as assoc" +				
		" where assoc.nome = 'CUT' and engenheiro.id < 99" ).setFirstResult(1).getResultList();


		System.err.println("consulta em relacionamento M-N: "+q.size());

	}



}
