/* 
This file is part of the PolePosition database benchmark
http://www.polepos.org

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package org.polepos.teams.jpa.data.modelo;

import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.ManyToOne;

import org.polepos.teams.jpa.data.JPATreeVisitor;

@Entity
public class JPATree {
    
    @Id public int id;
    @ManyToOne public JPATree preceding;
    @ManyToOne public JPATree subsequent;
    public String name;
    public int depth;
    public static int idGenerator = 0;
    
    public JPATree(){
    	 this.id = idGenerator;
         idGenerator += 1;
    }
    
    public JPATree(String name, int depth){
        this.name = name;
        this.depth = depth;
        this.id = idGenerator;
        idGenerator += 1;
    }
        
        
    public int getId(){
        return id;
    }
    
    public void setId(int id){
        this.id = id;;
    }

    public JPATree getPreceding(){
        return preceding;
    }
    
    public void setPreceding(JPATree tree){
        preceding = tree;
    }

    public JPATree getSubsequent(){
        return subsequent;
    }
    
    public void setSubsequent(JPATree tree){
        subsequent = tree;
    }
    
    public String getName(){
        return name;
    }
    
    public void setName(String name){
        this.name = name;
    }
    
    public int getDepth(){
        return depth;
    }
    
    public void setDepth(int depth){
        this.depth = depth;
    }
    
    public static JPATree createTree(int depth){
        return createTree(depth, 0);
    }
    
    private static JPATree createTree(int maxDepth, int currentDepth){
        
        if(maxDepth <= 0){
            return null;
        }
        
        JPATree tree = new JPATree();
        if(currentDepth == 0){
            tree.name = "root";
        }else{
            tree.name = "node at depth " + currentDepth;
        }
        tree.depth =  currentDepth;
        tree.preceding = createTree(maxDepth - 1, currentDepth + 1); 
        tree.subsequent = createTree(maxDepth - 1, currentDepth + 1); 
        return tree;
    }
    
    public static void traverse(JPATree tree, JPATreeVisitor visitor){
        if(tree == null){
            return;
        }
        traverse(tree.preceding, visitor);
        traverse(tree.subsequent, visitor);
        visitor.visit(tree);
    }

}
